/******************************************************************************/
/* SlunkCrypt, by LoRd_MuldeR <MuldeR2@GMX.de>                                */
/* This work has been released under the CC0 1.0 Universal license!           */
/******************************************************************************/

#ifdef _WIN32
#define _CRT_RAND_S 1
#define WIN32_LEAN_AND_MEAN 1
#endif

#include <slunkcrypt.h>

#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#include <fcntl.h>
#if (defined(__GLIBC__) && defined(__GLIBC_MINOR__) && (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 25)) || (defined(__FreeBSD__) && (__FreeBSD__ >= 12))
#define HAVE_GENRANDOM_SYSCALL 1
#include <sys/random.h>
#else
#undef HAVE_GENRANDOM_SYSCALL
#endif
#if (defined(__GLIBC__) && defined(__GLIBC_MINOR__) && (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 25)) || (defined(__FreeBSD__) && (__FreeBSD__ >= 11))
#define HAVE_EXPLICIT_BZERO 1
#include <string.h>
#else
#undef HAVE_EXPLICIT_BZERO
#endif
#endif

int slunkcrypt_random_bytes(uint8_t* const buffer, const size_t length)
{
#ifdef _WIN32
	size_t pos = 0U;
	while (pos < length)
	{
		const size_t bytes_left = length - pos;
		const size_t bytes_copy = (bytes_left < sizeof(uint32_t)) ? bytes_left : sizeof(uint32_t);
		uint32_t temp;
		if (rand_s(&temp) != 0)
		{
			return -1;
		}
		for (size_t i = 0; i < bytes_copy; ++i)
		{
			buffer[pos++] = (uint8_t)(temp & 0xFF);
			temp >>= 8;
		}
	}
	return 0;
#else
#ifdef HAVE_GENRANDOM_SYSCALL
	if (getrandom(buffer, length, 0U) >= length)
	{
		return 0;
	}
	return -1;
#else
	static const char* const PATH[] = { "/dev/urandom", "/dev/arandom", "/dev/random" };
	int result = -1;
	for (size_t i = 0; (i < 3U) && (result < 0); ++i)
	{
		const int fd = open(PATH[i], O_RDONLY);
		if (fd >= 0)
		{
			if (read(fd, buffer, length) >= length)
			{
				result = 0;
			}
			close(fd);
		}
	}
	return result;
#endif
#endif
}

void slunkcrypt_bzero(void* const ptr, const size_t length)
{
	if ((ptr) && (length > 0U))
	{
#ifdef _WIN32
		SecureZeroMemory(ptr, length);
#else
#ifdef HAVE_EXPLICIT_BZERO
		explicit_bzero(ptr, length);
#else
		volatile uint8_t* buffer = ptr;
		for (size_t i = 0U; i < length; ++i)
		{
			buffer[i] = 0U;
		}
#endif
#endif
	}
}
